@@ -266,7 +266,7 @@ soup_auth_update (SoupAuth *auth, SoupMessage *msg, const char *auth_header)
 	g_return_val_if_fail (auth_header != NULL, FALSE);
 
 	scheme = soup_auth_get_scheme_name (auth);
-	if (strncmp (auth_header, scheme, strlen (scheme)) != 0)
+	if (g_ascii_strncasecmp (auth_header, scheme, strlen (scheme)) != 0)
 		return FALSE;
 
 	params = soup_header_parse_param_list (auth_header + strlen (scheme));
@@ -302,7 +302,8 @@ soup_auth_authenticate (SoupAuth *auth, const char *username, const char *passwo
 	gboolean was_authenticated;
 
 	g_return_if_fail (SOUP_IS_AUTH (auth));
-	g_return_if_fail (username != NULL || password == NULL);
+	g_return_if_fail (username != NULL);
+	g_return_if_fail (password != NULL);
 
 	was_authenticated = soup_auth_is_authenticated (auth);
 	SOUP_AUTH_GET_CLASS (auth)->authenticate (auth, username, password);
