@@ -490,7 +490,7 @@ soup_dns_lookup_address (struct sockaddr *sockaddr)
 
 	entry = soup_dns_cache_entry_lookup (name);
 	if (!entry)
-		entry = soup_dns_cache_entry_new (name); // FIXME
+		entry = soup_dns_cache_entry_new (name); /* FIXME */
 	g_free (name);
 
 	lookup = g_slice_new0 (SoupDNSLookup);
@@ -520,10 +520,10 @@ do_async_callback (gpointer user_data)
 	SoupDNSCacheEntry *entry = lookup->entry;
 	GCancellable *cancellable = lookup->cancellable;
 
-	lookup->callback (lookup, resolve_status (entry, cancellable),
-			  lookup->user_data);
 	if (cancellable)
 		g_signal_handlers_disconnect_by_func (cancellable, async_cancel, lookup);
+	lookup->callback (lookup, resolve_status (entry, cancellable),
+			  lookup->user_data);
 
 	return FALSE;
 }
