@@ -33,7 +33,6 @@
 
 typedef void (*SoupHeaderSetter) (SoupMessageHeaders *, const char *);
 static const char *intern_header_name (const char *name, SoupHeaderSetter *setter);
-static void clear_special_headers (SoupMessageHeaders *hdrs);
 
 typedef struct {
 	const char *name;
@@ -99,6 +98,7 @@ soup_message_headers_free (SoupMessageHeaders *hdrs)
 		g_array_free (hdrs->array, TRUE);
 		if (hdrs->concat)
 			g_hash_table_destroy (hdrs->concat);
+		g_free (hdrs->content_type);
 		g_slice_free (SoupMessageHeaders, hdrs);
 	}
 }
@@ -137,7 +137,7 @@ soup_message_headers_clear (SoupMessageHeaders *hdrs)
 	if (hdrs->concat)
 		g_hash_table_remove_all (hdrs->concat);
 
-	clear_special_headers (hdrs);
+	hdrs->encoding = -1;
 }
 
 /**
@@ -146,13 +146,7 @@ soup_message_headers_clear (SoupMessageHeaders *hdrs)
  * @name: the header name to add
  * @value: the new value of @name
  *
- * Appends a new header with name @name and value @value to @hdrs. (If
- * there is an existing header with name @name, then this creates a
- * second one, which is only allowed for list-valued headers; see also
- * soup_message_headers_replace().)
- *
- * The caller is expected to make sure that @name and @value are
- * syntactically correct.
+ * Appends a new header with name @name and value @value to @hdrs.
  **/
 void
 soup_message_headers_append (SoupMessageHeaders *hdrs,
@@ -164,25 +158,6 @@ soup_message_headers_append (SoupMessageHeaders *hdrs,
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (value != NULL);
 
-	/* Setting a syntactically invalid header name or value is
-	 * considered to be a programming error. However, it can also
-	 * be a security hole, so we want to fail here even if
-	 * compiled with G_DISABLE_CHECKS.
-	 */
-#ifndef G_DISABLE_CHECKS
-	g_return_if_fail (*name && strpbrk (name, " \t\r\n:") == NULL);
-	g_return_if_fail (strpbrk (value, "\r\n") == NULL);
-#else
-	if (*name && strpbrk (name, " \t\r\n:")) {
-		g_warning ("soup_message_headers_append: Ignoring bad name '%s'", name);
-		return;
-	}
-	if (strpbrk (value, "\r\n")) {
-		g_warning ("soup_message_headers_append: Ignoring bad value '%s'", value);
-		return;
-	}
-#endif
-
 	header.name = intern_header_name (name, &setter);
 	header.value = g_strdup (value);
 	g_array_append_val (hdrs->array, header);
@@ -198,11 +173,7 @@ soup_message_headers_append (SoupMessageHeaders *hdrs,
  * @name: the header name to replace
  * @value: the new value of @name
  *
- * Replaces the value of the header @name in @hdrs with @value. (See
- * also soup_message_headers_append().)
- *
- * The caller is expected to make sure that @name and @value are
- * syntactically correct.
+ * Replaces the value of the header @name in @hdrs with @value.
  **/
 void
 soup_message_headers_replace (SoupMessageHeaders *hdrs,
@@ -226,20 +197,6 @@ find_header (SoupHeader *hdr_array, const char *interned_name, int nth)
 	return -1;
 }
 
-static int
-find_last_header (SoupHeader *hdr_array, guint length, const char *interned_name, int nth)
-{
-	int i;
-
-	for (i = length; i >= 0; i--) {
-		if (hdr_array[i].name == interned_name) {
-			if (nth-- == 0)
-				return i;
-		}
-	}
-	return -1;
-}
-
 /**
  * soup_message_headers_remove:
  * @hdrs: a #SoupMessageHeaders
@@ -269,64 +226,22 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
 }
 
 /**
- * soup_message_headers_get_one:
+ * soup_message_headers_get:
  * @hdrs: a #SoupMessageHeaders
  * @name: header name
  * 
- * Gets the value of header @name in @hdrs. Use this for headers whose
- * values are <emphasis>not</emphasis> comma-delimited lists, and
- * which therefore can only appear at most once in the headers. For
- * list-valued headers, use soup_message_headers_get_list().
+ * Gets the value of header @name in @hdrs.
  *
- * If @hdrs does erroneously contain multiple copies of the header, it
- * is not defined which one will be returned. (Ideally, it will return
- * whichever one makes libsoup most compatible with other HTTP
- * implementations.)
- *
- * Return value: the header's value or %NULL if not found.
- *
- * Since: 2.26.1
- **/
-const char *
-soup_message_headers_get_one (SoupMessageHeaders *hdrs, const char *name)
-{
-	SoupHeader *hdr_array = (SoupHeader *)(hdrs->array->data);
-	guint hdr_length = hdrs->array->len;
-	int index;
-
-	g_return_val_if_fail (name != NULL, NULL);
-
-	name = intern_header_name (name, NULL);
-
-	index = find_last_header (hdr_array, hdr_length, name, 0);
-
-	return (index == -1) ? NULL : hdr_array[index].value;
-}
-
-/**
- * soup_message_headers_get_list:
- * @hdrs: a #SoupMessageHeaders
- * @name: header name
- * 
- * Gets the value of header @name in @hdrs. Use this for headers whose
- * values are comma-delimited lists, and which are therefore allowed
- * to appear multiple times in the headers. For non-list-valued
- * headers, use soup_message_headers_get_one().
- *
- * If @name appears multiple times in @hdrs,
- * soup_message_headers_get_list() will concatenate all of the values
- * together, separated by commas. This is sometimes awkward to parse
- * (eg, WWW-Authenticate, Set-Cookie), but you have to be able to deal
- * with it anyway, because the HTTP spec explicitly states that this
- * transformation is allowed, and so an upstream proxy could do the
- * same thing.
+ * If @name has multiple values in @hdrs, soup_message_headers_get()
+ * will concatenate all of the values together, separated by commas.
+ * This is sometimes awkward to parse (eg, WWW-Authenticate,
+ * Set-Cookie), but you have to be able to deal with it anyway,
+ * because an upstream proxy could do the same thing.
  * 
  * Return value: the header's value or %NULL if not found.
- *
- * Since: 2.26.1
  **/
 const char *
-soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
+soup_message_headers_get (SoupMessageHeaders *hdrs, const char *name)
 {
 	SoupHeader *hdr_array = (SoupHeader *)(hdrs->array->data);
 	GString *concat;
@@ -362,32 +277,6 @@ soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
 	return value;
 }
 
-/**
- * soup_message_headers_get:
- * @hdrs: a #SoupMessageHeaders
- * @name: header name
- * 
- * Gets the value of header @name in @hdrs.
- *
- * This method was supposed to work correctly for both single-valued
- * and list-valued headers, but because some HTTP clients/servers
- * mistakenly send multiple copies of headers that are supposed to be
- * single-valued, it sometimes returns incorrect results. To fix this,
- * the methods soup_message_headers_get_one() and
- * soup_message_headers_get_list() were introduced, so callers can
- * explicitly state which behavior they are expecting.
- *
- * Return value: as with soup_message_headers_get_list().
- * 
- * Deprecated: Use soup_message_headers_get_one() or
- * soup_message_headers_get_list() instead.
- **/
-const char *
-soup_message_headers_get (SoupMessageHeaders *hdrs, const char *name)
-{
-	return soup_message_headers_get_list (hdrs, name);
-}
-
 /**
  * SoupMessageHeadersIter:
  *
@@ -548,22 +437,6 @@ intern_header_name (const char *name, SoupHeaderSetter *setter)
 	return interned;
 }
 
-static void
-clear_special_headers (SoupMessageHeaders *hdrs)
-{
-	SoupHeaderSetter setter;
-	GHashTableIter iter;
-	gpointer key, value;
-
-	/* Make sure header_setters has been initialized */
-	intern_header_name ("", NULL);
-
-	g_hash_table_iter_init (&iter, header_setters);
-	while (g_hash_table_iter_next (&iter, &key, &value)) {
-		setter = value;
-		setter (hdrs, NULL);
-	}
-}
 
 /* Specific headers */
 
@@ -635,20 +508,13 @@ soup_message_headers_get_encoding (SoupMessageHeaders *hdrs)
 	/* If Transfer-Encoding was set, hdrs->encoding would already
 	 * be set. So we don't need to check that possibility.
 	 */
-	header = soup_message_headers_get_one (hdrs, "Content-Length");
+	header = soup_message_headers_get (hdrs, "Content-Length");
 	if (header) {
 		content_length_setter (hdrs, header);
 		if (hdrs->encoding != -1)
 			return hdrs->encoding;
 	}
 
-	/* Per RFC 2616 4.4, a response body that doesn't indicate its
-	 * encoding otherwise is terminated by connection close, and a
-	 * request that doesn't indicate otherwise has no body. Note
-	 * that SoupMessage calls soup_message_headers_set_encoding()
-	 * to override the response body default for our own
-	 * server-side messages.
-	 */
 	hdrs->encoding = (hdrs->type == SOUP_MESSAGE_HEADERS_RESPONSE) ?
 		SOUP_ENCODING_EOF : SOUP_ENCODING_NONE;
 	return hdrs->encoding;
@@ -822,8 +688,6 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
  * If @end is %-1 and @start is negative, then it represents a "suffix
  * range", referring to the last -@start bytes of the resource body.
  * (Eg, the last 500 bytes would be @start = %-500 and @end = %-1.)
- *
- * Since: 2.26
  **/
 
 static int
@@ -856,8 +720,6 @@ sort_ranges (gconstpointer a, gconstpointer b)
  * Return value: %TRUE if @hdrs contained a "Range" header containing
  * byte ranges which could be parsed, %FALSE otherwise (in which case
  * @range and @length will not be set).
- *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
@@ -865,7 +727,7 @@ soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
 				 SoupRange          **ranges,
 				 int                 *length)
 {
-	const char *range = soup_message_headers_get_one (hdrs, "Range");
+	const char *range = soup_message_headers_get (hdrs, "Range");
 	GSList *range_list, *r;
 	GArray *array;
 	char *spec, *end;
@@ -940,8 +802,6 @@ soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
  * @ranges: an array of #SoupRange
  *
  * Frees the array of ranges returned from soup_message_headers_get_ranges().
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
@@ -959,8 +819,6 @@ soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
  * Sets @hdrs's Range header to request the indicated ranges. (If you
  * only want to request a single range, you can use
  * soup_message_headers_set_range().)
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
@@ -1001,8 +859,6 @@ soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
  *
  * If you need to request multiple ranges, use
  * soup_message_headers_set_ranges().
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
@@ -1030,8 +886,6 @@ soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
  *
  * Return value: %TRUE if @hdrs contained a "Content-Range" header
  * containing a byte range which could be parsed, %FALSE otherwise.
- *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
@@ -1039,7 +893,7 @@ soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
 					goffset             *end,
 					goffset             *total_length)
 {
-	const char *header = soup_message_headers_get_one (hdrs, "Content-Range");
+	const char *header = soup_message_headers_get (hdrs, "Content-Range");
 	goffset length;
 	char *p;
 
@@ -1080,8 +934,6 @@ soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
  * Sets @hdrs's Content-Range header according to the given values.
  * (Note that @total_length is the total length of the entire resource
  * that this is a range of, not simply @end - @start + 1.)
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_range (SoupMessageHeaders  *hdrs,
@@ -1110,7 +962,7 @@ parse_content_foo (SoupMessageHeaders *hdrs, const char *header_name,
 	const char *header;
 	char *semi;
 
-	header = soup_message_headers_get_one (hdrs, header_name);
+	header = soup_message_headers_get (hdrs, header_name);
 	if (!header)
 		return FALSE;
 
@@ -1168,15 +1020,8 @@ content_type_setter (SoupMessageHeaders *hdrs, const char *value)
 {
 	g_free (hdrs->content_type);
 	if (value) {
-		char *content_type, *p;
-
-		parse_content_foo (hdrs, "Content-Type", &content_type, NULL);
-		p = strpbrk (content_type, " /");
-		if (!p || *p != '/' || strpbrk (p + 1, " /")) {
-			g_free (content_type);
-			hdrs->content_type = NULL;
-		} else
-			hdrs->content_type = content_type;
+		parse_content_foo (hdrs, "Content-Type",
+				   &hdrs->content_type, NULL);
 	} else
 		hdrs->content_type = NULL;
 }
@@ -1191,18 +1036,16 @@ content_type_setter (SoupMessageHeaders *hdrs, const char *value)
  * its value in *@content_type and *@params. @params can be %NULL if you
  * are only interested in the content type itself.
  *
- * Return value: a string with the value of the "Content-Type" header
- * or NULL if @hdrs does not contain that header or it cannot be
- * parsed (in which case *@params will be unchanged).
- *
- * Since: 2.26
+ * Return value: %TRUE if @hdrs contains a "Content-Type" header,
+ * %FALSE if not (in which case *@content_type and *@params will be
+ * unchanged).
  **/
 const char *
 soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
 				       GHashTable         **params)
 {
 	if (!hdrs->content_type)
-		return NULL;
+		return FALSE;
 
 	if (params)
 		parse_content_foo (hdrs, "Content-Type", NULL, params);
@@ -1217,8 +1060,6 @@ soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
  *
  * Sets the "Content-Type" header in @hdrs to @content_type,
  * optionally with additional parameters specified in @params.
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
@@ -1255,8 +1096,6 @@ soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
  * Return value: %TRUE if @hdrs contains a "Content-Disposition"
  * header, %FALSE if not (in which case *@disposition and *@params
  * will be unchanged).
- *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
@@ -1293,8 +1132,6 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
  *
  * See soup_message_headers_get_content_disposition() for a discussion
  * of how Content-Disposition is used in HTTP.
- *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_disposition (SoupMessageHeaders  *hdrs,
